#!/usr/bin/env python
# vim: set fileencoding=utf8:
"""
Decorators of object-permission

AUTHOR:
    lambdalisue[Ali su ae] (lambdalisue@hashnote.net)
    
Copyright:
    Copyright 2011 Alisue allright reserved.

License:
    Licensed under the Apache License, Version 2.0 (the "License"); 
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unliss required by applicable law or agreed to in writing, software
    distributed under the License is distrubuted on an "AS IS" BASICS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""
__AUTHOR__ = "lambdalisue (lambdalisue@hashnote.net)"
from django.http import HttpRequest
from django.http import HttpResponseForbidden
from django.views.generic.edit import CreateView
from django.views.generic.detail import SingleObjectMixin
from django.contrib.auth.views import redirect_to_login

from utils import _get_object_from_list_detail_object_detail
from utils import _get_object_from_date_based_object_detail

def permission_required(perm, queryset=None):
    """
    Permission check decorator for generic view
    
    Arguments:
        perm     - permission code same format as Django's permission code
                   when you need ObjectPermission feature only, you can cut
                   off `app_label` and `model` See Usage below.
        queryset - QuerySet or Model object for filtering.
                   With Classbased generic view, None for using class default
                   queryset if the class have or None for non object permission
                   check.
                   With Functional generic view, None for non object permission
    Note:
        Using Classbased generic view is recommended while you can regulate the
        queryset used with class method in Classbased generic view. Getting object
        from parameters of functional generic view might not work correctly sometime.

    Warning:
        DO NOT USE 'django.utils.decorators.method_decorator'. This decorator
        can be used either Classbased/Functional generic view without the 
        method_decorator

    """
    def wrapper(fn):
        def inner(self, request, *args, **kwargs):
            if isinstance(self, HttpRequest):
                # functional based generic view
                if request is not None:
                    args.insert(0, request)
                request = self
                self = None
            if self is None:
                if queryset:
                    kwargs['queryset'] = queryset
                    # functional based generic view
                    if 'date_field' in kwargs:
                        # assume that fn is date_based.object_detail
                        obj = _get_object_from_date_based_object_detail(
                                request, *args, **kwargs)
                    else:
                        # assume that fn is list_detail.object_detail
                        obj = _get_object_from_list_detail_object_detail(
                                request, *args, **kwargs)
                else:
                    obj = None
            else:
                # classbased generic view
                if not isinstance(self, CreateView) and isinstance(self, SingleObjectMixin):
                    self.kwargs = kwargs
                    obj = self.get_object(queryset)
                else:
                    obj = None
            if not request.user.has_perm(perm, obj):
                if request.user.is_authenticated():
                    return HttpResponseForbidden()
                else:
                    return redirect_to_login(request.path)
            if self is None:
                return fn(request, *args, **kwargs)
            else:
                return fn(self, request, *args, **kwargs)
        return inner
    return wrapper



#!/usr/bin/env python
# vim: set fileencoding=utf8:
"""
Authentication Backend for 'django.contrib.auth' app

Add this backend to your ``AUTHENTICATION_BACKENDS`` like below::

    AUTHENTICATION_BACKENDS = (
        'django.contrib.auth.backends.ModelBackend',
        'object_permission.backends.ObjectPermBackend',
    )


AUTHOR:
    lambdalisue[Ali su ae] (lambdalisue@hashnote.net)
    
Copyright:
    Copyright 2011 Alisue allright reserved.

License:
    Licensed under the Apache License, Version 2.0 (the "License"); 
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unliss required by applicable law or agreed to in writing, software
    distributed under the License is distrubuted on an "AS IS" BASICS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""
__AUTHOR__ = "lambdalisue (lambdalisue@hashnote.net)"
from django.db.models import Model
from django.contrib.contenttypes.models import ContentType

from models import UserObjectPermission
from models import GroupObjectPermission
from models import AnonymousObjectPermission

from utils import get_perm_codename

class ObjectPermBackend(object):
    """Authentication backend for object-permission"""
    supports_object_permissions = True
    supports_anonymous_user = True

    def authenticate(self, username, password):
        """This backend is only for checking permission"""
        return None

    def _has_user_object_permission(cls, user_obj, perm_codename, obj):
        """check permission of obj for user from UserObjectPermission"""
        if user_obj and not user_obj.is_authenticated():
            return False
        ct = ContentType.objects.get_for_model(obj)
        qs = UserObjectPermission.objects.filter(
                content_type=ct,
                object_id=obj.pk,
                user=user_obj)
        qs = qs.filter(permissions__codename=perm_codename).distinct()
        return qs.exists()

    def _has_group_object_permission(cls, user_obj, perm_codename, obj):
        """check permission of obj for user from GroupObjectPermission"""
        if not user_obj.is_authenticated():
            return False
        ct = ContentType.objects.get_for_model(obj)
        qs = GroupObjectPermission.objects.filter(
                content_type=ct,
                object_id=obj.pk,
                group__in=user_obj.groups.all())
        qs = qs.filter(permissions__codename=perm_codename).distinct()
        return qs.exists()

    def _has_anonymous_object_permission(cls, user_obj, perm_codename, obj):
        """check permission of obj for anonymous from AnonymousObjectPermission"""
        if user_obj.is_authenticated():
            return False
        ct = ContentType.objects.get_for_model(obj)
        qs = AnonymousObjectPermission.objects.filter(
                content_type=ct,
                object_id=obj.pk)
        qs = qs.filter(permissions__codename=perm_codename).distinct()
        return qs.exists()

    def has_perm(self, user_obj, perm, obj=None):
        """check permission of obj for user_obj"""
        if obj is None or not isinstance(obj, Model):
            # This is object permission backend so don't touch if obj is None
            return False
        
        perm_codename = get_perm_codename(perm)
            
        if user_obj.is_authenticated():
            # check user_obj specific permissions
            if self._has_user_object_permission(user_obj, perm_codename, obj):
                return True
            # check authenticated user permissions
            elif self._has_user_object_permission(None, perm_codename, obj):
                return True
            # check groups user_obj belong specific permissions
            elif self._has_group_object_permission(user_obj, perm_codename, obj):
                return True
        # check anonymous user specific permissions
        elif self._has_anonymous_object_permission(user_obj, perm_codename, obj):
            return True
        return False



#!/usr/bin/env python
# vim: set fileencoding=utf8:
"""
object-permission mediator module

mediator is to use add/remove permissions to particular object


AUTHOR:
    lambdalisue[Ali su ae] (lambdalisue@hashnote.net)
    
Copyright:
    Copyright 2011 Alisue allright reserved.

License:
    Licensed under the Apache License, Version 2.0 (the "License"); 
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unliss required by applicable law or agreed to in writing, software
    distributed under the License is distrubuted on an "AS IS" BASICS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""
__AUTHOR__ = "lambdalisue (lambdalisue@hashnote.net)"
from django.db.models import Model
from django.contrib.auth.models import User
from django.contrib.auth.models import Group
from django.contrib.auth.models import Permission
from django.contrib.auth.models import AnonymousUser
from django.contrib.contenttypes.models import ContentType

from models import UserObjectPermission
from models import GroupObjectPermission
from models import AnonymousObjectPermission
from utils import get_perm_codename
from utils import get_perm_codename_with_suffix

class ObjectPermMediatorBase(object):
    """Base Mediator for object-permission"""

    def __init__(self, obj):
        """constructor for ObjectPermMediator

        Attribute:
            obj - a target model instance of object permission
        """
        if not isinstance(obj, Model):
            raise AttributeError("'%s' is not an instance of model" % obj)
        self._obj = obj
        self._ct = ContentType.objects.get_for_model(obj)

    def _get_or_create_permission(self, perm):
        """get or create django permission instance"""
        perm_codename = get_perm_codename(perm)
        try:
            # try to find permissin without suffix
            permission = Permission.objects.get(
                    content_type=self._ct,
                    codename=perm_codename)
        except Permission.DoesNotExist:
            # try to find/create permission with suffix
            perm_codename = get_perm_codename_with_suffix(perm, self._obj)
            permission = Permission.objects.get_or_create(
                    content_type=self._ct,
                    codename=perm_codename)[0]
        return permission

    def _get_object_permission_cls(self, instance):
        """get object permission cls suite for instance"""
        ANONYMOUS = 'anonymous'
        if isinstance(instance, basestring) and instance == ANONYMOUS:
            return AnonymousObjectPermission, {}
        elif isinstance(instance, AnonymousUser):
            return AnonymousObjectPermission, {}
        elif isinstance(instance, Group):
            return GroupObjectPermission, {'group': instance}
        elif instance is None or isinstance(instance, User) or isinstance(instance, basestring):
            if isinstance(instance, basestring):
                # Search from username
                try:
                    instance = User.objects.get(username=instance)
                except User.DoesNotExist:
                    raise AttributeError("Unknown parameter '%s' is passed, you may mean '%s'?" % (instance, ANONYMOUS))
            return UserObjectPermission, {'user': instance}
        raise AttributeError("Unknown parameter '%s' is passed" % instance)

    def _get_or_create_object_permission(self, instance):
        """get or create object permission suite for instance"""
        model, kwargs = self._get_object_permission_cls(instance)
        return model.objects.get_or_create_object_permission(self._obj, **kwargs)[0]

    def reset(self):
        """reset all permissions of obj"""
        AnonymousObjectPermission.objects.get_for_model(self._obj).remove()
        GroupObjectPermission.objects.get_for_model(self._obj).remove()
        UserObjectPermission.objects.get_for_model(self._obj).remove()

    def clear(self, instance_or_iterable):
        """clear all object permissions of obj for instance(s)"""
        if not hasattr(instance_or_iterable, '__iter__'):
            instance_or_iterable = [instance_or_iterable]
        for instance in instance_or_iterable:
            object_permission = self._get_or_create_object_permission(instance)
            object_permission.permissions.clear()

    def contribute(self, instance_or_iterable, permissions=[]):
        """contribute permissions of obj to instance(s)

        instances can be an instance of User, Group, AnonymousUser or 'anonymous'
        for AnonymousUser shortcut and None for All authenticated user.

        Attribute:
            instance_or_iterable - User, Group, AnonymousUser to cotribute permission.
                                   'anonymous' for anonymous user and None for all
                                   authenticated users
            permissions          - codename list of permission
        """
        if not hasattr(instance_or_iterable, '__iter__'):
            instance_or_iterable = [instance_or_iterable]
        for instance in instance_or_iterable:
            object_permission = self._get_or_create_object_permission(instance)
            for perm in permissions:
                permission = self._get_or_create_permission(perm)
                object_permission.permissions.add(permission)

    def discontribute(self, instance_or_iterable, permissions=[]):
        """discontribute permissions of obj to instance(s)

        instances can be an instance of User, Group, AnonymousUser or 'anonymous'
        for AnonymousUser shortcut and None for All authenticated user.

        Attribute:
            instance_or_iterable - User, Group, AnonymousUser to cotribute permission.
                                   'anonymous' for anonymous user and None for all
                                   authenticated users
            permissions          - codename list of permission
        """
        if not hasattr(instance_or_iterable, '__iter__'):
            instance_or_iterable = [instance_or_iterable]
        for instance in instance_or_iterable:
            object_permission = self._get_or_create_object_permission(instance)
            for perm in permissions:
                permission = self._get_or_create_permission(perm)
                object_permission.permissions.remove(permission)

class ObjectPermMediator(ObjectPermMediatorBase):
    """Mediator class for object permission"""
    def _contribute(self, instance_or_iterable, permissions, extra_permissions):
        if isinstance(extra_permissions, (list, tuple)):
            permissions += list(extra_permissions)
        self.clear(instance_or_iterable)
        self.contribute(instance_or_iterable, permissions)

    def reject(self, instance_or_iterable, extra_permissions=[]):
        """reject all management permission (view, change, delete)"""
        permissions = []
        self._contribute(instance_or_iterable, permissions, extra_permissions)
    def viewer(self, instance_or_iterable, extra_permissions=[]):
        """can view"""
        permissions = ['view']
        self._contribute(instance_or_iterable, permissions, extra_permissions)
    def ediinstance_or_iterabler(self, instance_or_iterable, extra_permissions=[]):
        """can view and change"""
        permissions = ['view', 'change']
        self._contribute(instance_or_iterable, permissions, extra_permissions)
    def manager(self, instance_or_iterable, extra_permissions=[]):
        """can view, change and delete"""
        permissions = ['view', 'change', 'delete']
        self._contribute(instance_or_iterable, permissions, extra_permissions)



#!/usr/bin/env python
# vim: set fileencoding=utf8:
"""
templatetag for django-object-permission

Django 1.2 template tag that supports {% elif %} branches and
'of' operator for checking object permission.


AUTHOR:
    lambdalisue[Ali su ae] (lambdalisue@hashnote.net)
    
Copyright:
    Copyright 2011 Alisue allright reserved.

License:
    Licensed under the Apache License, Version 2.0 (the "License"); 
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unliss required by applicable law or agreed to in writing, software
    distributed under the License is distrubuted on an "AS IS" BASICS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""
__AUTHOR__ = "lambdalisue (lambdalisue@hashnote.net)"
import warnings

from django import template
from django.template import Node, VariableDoesNotExist
from django.template.smartif import infix
from django.template.smartif import IfParser
from django.template.smartif import OPERATORS as _OPERATORS
from django.template.defaulttags import TemplateLiteral

register = template.Library()

OPERATORS = dict(_OPERATORS,
    of=infix(10, lambda context, x, y: 
        template.resolve_variable('request', context).user.has_perm(x.eval(context), y.eval(context)))
)

class ObjectPermissionIfParser(IfParser):
    def translate_token(self, token):
        try:
            op = OPERATORS[token]
        except (KeyError, TypeError):
            return self.create_var(token)
        else:
            return op()
class TemplateObjectPermissionIfParser(ObjectPermissionIfParser):
    error_class = template.TemplateSyntaxError

    def __init__(self, parser, *args, **kwargs):
        self.template_parser = parser
        return super(TemplateObjectPermissionIfParser, self).__init__(*args, **kwargs)

    def create_var(self, value):
        return TemplateLiteral(self.template_parser.compile_filter(value), value)
    
class IfBranch(object):
    def __init__(self, var, node_list):
        self.var = var
        self.node_list = node_list

class IfNode(Node):
    def __init__(self, branches):
        self.branches = branches

    def __repr__(self):
        return "<If node>"

    def __iter__(self):
        for n in self.branches:
            for node in n:
                yield node

    def render(self, context):
        for n in self.branches:
            var = n.var
            if var != True:
                try:
                    var = var.eval(context)
                except VariableDoesNotExist:
                    var = None
            if var:
                return n.node_list.render(context)
                break
        return ""

@register.tag('ifhsp')
def do_ifhsp(parser, token):
    warnings.warn('deprecated. use `pif` insted', DeprecationWarning)
    return do_if(parser, token)
@register.tag('pif')
def do_if(parser, token):
    """
    Django 1.2 template tag that supports {% elif %} branches and
    'of' operator for checking object permission.

    .. Warning::
        This template tag assumed that context has 'request' so make sure your
        ``TEMPLATE_CONTEXTS`` is like below::
        
            TEMPLATE_CONTEXT_PROCESSORS = (
                "django.core.context_processors.auth",           # This one is required
                "django.core.context_processors.debug",
                "django.core.context_processors.i18n",
                "django.core.context_processors.media",
                "django.core.context_processors.request",        # This one is required
            )

    Usage::

        {% pif 'blogs.add_entry' of None or user.is_staff %}
            You can add post
        {% elif 'blogs.change_entry' of object or 'blogs.delete_entry' of object %}
            You can update/delete this entry
        {% endpif %}
    """
    class Enders(list):
        def __init__(self, endtag):
            self.endtag = endtag
        def __contains__(self, val):
            return val.startswith('elif') or val in ('else', self.endtag)
    
    name = contents = token.split_contents()[0]
    endtag = "end%s" % name
    enders = Enders(endtag)
    branches = []
    
    while True:
        contents = token.split_contents()
        bits = contents[1:]
        if contents[0] in (name, "elif"):
            var = TemplateObjectPermissionIfParser(parser, bits).parse()
            nodelist = parser.parse(enders)
            next_token = parser.next_token()
            branches.append(IfBranch(var, nodelist))
            token = next_token
        elif token.contents == 'else':
            nodelist = parser.parse((endtag,))
            parser.delete_first_token()
            branches.append(IfBranch(True, nodelist))
            break
        elif token.contents == endtag:
            break

    return IfNode(branches)



#!/usr/bin/env python
# vim: set fileencoding=utf8:
"""
Default Object permission handler for model


AUTHOR:
    lambdalisue[Ali su ae] (lambdalisue@hashnote.net)
    
Copyright:
    Copyright 2011 Alisue allright reserved.

License:
    Licensed under the Apache License, Version 2.0 (the "License"); 
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unliss required by applicable law or agreed to in writing, software
    distributed under the License is distrubuted on an "AS IS" BASICS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""
__AUTHOR__ = "lambdalisue (lambdalisue@hashnote.net)"
from django.db.models.signals import post_save
from django.db.models.signals import post_delete
from django.contrib.auth.models import User
from base import ObjectPermHandlerBase

class AuthenticatedObjectPermHandler(ObjectPermHandlerBase):
    """ObjectPermHandler for model

    This handler contribute..

        1.  Manager permission to authenticated staff user
        2.  Editor permission to authenticated user
        3.  Viewer permission to anonymous user

    """
    def __init__(self, model):
        super(AuthenticatedObjectPermHandler, self).__init__(model)
        # Register this instance to class
        cls = self.__class__
        if not hasattr(cls, '_instances'):
            cls._instances = []
        cls._instances.append(self)

    def created(self):
        # staff user has full access
        staff_users = User.objects.filter(staff=True)
        self.mediator.manager(staff_users.iterator())
        # Authenticated user can edit
        self.mediator.editor(None)
        # Anonymous user can view
        self.mediator.viewer('anonymous')

def _recall_updated_reciver(sender, instance, **kwargs):
    # Recall updated method of all AuthenticatedObjectPermHandler instance
    instances = getattr(AuthenticatedObjectPermHandler, '_instances', [])
    for instance in instances:
        instance.mediator.reset()
        instance.updated()
post_save.connect(_recall_updated_reciver, User)
post_delete.connect(_recall_updated_reciver, User)



#!/usr/bin/env python
# vim: set fileencoding=utf8:
"""
short module explanation


AUTHOR:
    lambdalisue[Ali su ae] (lambdalisue@hashnote.net)
    
Copyright:
    Copyright 2011 Alisue allright reserved.

License:
    Licensed under the Apache License, Version 2.0 (the "License"); 
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unliss required by applicable law or agreed to in writing, software
    distributed under the License is distrubuted on an "AS IS" BASICS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""
__AUTHOR__ = "lambdalisue (lambdalisue@hashnote.net)"
from django.conf import settings
from django.core.exceptions import ImproperlyConfigured
from django.utils.importlib import import_module

def load_handler_class(path):
    i = path.rfind('.')
    module, attr = path[:i], path[i+1:]
    try:
        mod = import_module(module)
    except ImportError, e:
        raise ImproperlyConfigured('Error importing object permission handler %s: "%s"' % (path, e))
    try:
        cls = getattr(mod, attr)
    except AttributeError:
        raise ImproperlyConfigured('Module "%s" does not define a "%s" object permission handler' % (module, attr))

    return cls

if isinstance(settings.OBJECT_PERMISSION_DEFAULT_HANDLER_CLASS, basestring):
    settings.OBJECT_PERMISSION_DEFAULT_HANDLER_CLASS = \
            load_handler_class(settings.OBJECT_PERMISSION_DEFAULT_HANDLER_CLASS)

from base import ObjectPermHandlerBase
from authenticated import AuthenticatedObjectPermHandler



#!/usr/bin/env python
# vim: set fileencoding=utf8:
"""
Base module of object-permission handler


AUTHOR:
    lambdalisue[Ali su ae] (lambdalisue@hashnote.net)
    
Copyright:
    Copyright 2011 Alisue allright reserved.

License:
    Licensed under the Apache License, Version 2.0 (the "License"); 
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unliss required by applicable law or agreed to in writing, software
    distributed under the License is distrubuted on an "AS IS" BASICS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""
__AUTHOR__ = "lambdalisue (lambdalisue@hashnote.net)"
from django.db.models.signals import post_save
from django.db.models.signals import post_delete

from ..mediators import ObjectPermMediator
class ObjectPermHandlerBase(object):
    """Base ObjectPermHandler"""
    def __init__(self, model):
        self.model = model

    def bind(self):
        """bind signals and recivers"""
        post_save.connect(self._post_save_reciever, sender=self.model, weak=False)
        post_delete.connect(self._post_delete_reciver, sender=self.model, weak=False)

    def unbind(self):
        """unbind signals and recivers."""
        post_save.disconnect(self._post_save_reciever, sender=self.model)
        post_delete.disconnect(self._post_delete_reciver, sender=self.model)

    def _post_save_reciever(self, sender, instance, created, **kwargs):
        self.obj = instance
        self.mediator = ObjectPermMediator(self.obj)
        if created:
            self.created()
        else:
            self.updated()
    def _post_delete_reciver(self, sender, instance, **kwargs):
        self.obj = instance
        self.mediator = ObjectPermMediator(self.obj)
        self.deleted()


    def created(self):
        """called when object is created"""
        pass

    def updated(self):
        """called when object is updated"""
        pass
    
    def deleted(self):
        """called when object is deleted"""
        pass




#!/usr/bin/env python
# vim: set fileencoding=utf8:
"""
initialization django-object-permission

Add this backend to your ``AUTHENTICATION_BACKENDS`` like below::

    AUTHENTICATION_BACKENDS = (
        'django.contrib.auth.backends.ModelBackend',
        'object_permission.backends.ObjectPermBackend',
    )

AUTHOR:
    lambdalisue[Ali su ae] (lambdalisue@hashnote.net)
    
Copyright:
    Copyright 2011 Alisue allright reserved.

License:
    Licensed under the Apache License, Version 2.0 (the "License"); 
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unliss required by applicable law or agreed to in writing, software
    distributed under the License is distrubuted on an "AS IS" BASICS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""
__AUTHOR__ = "lambdalisue (lambdalisue@hashnote.net)"
from django.conf import settings
from django.db.models import get_models, signals
from django.contrib.auth.management import _get_permission_codename
from django.core.exceptions import ImproperlyConfigured

app_label = 'object_permission'
def set_default(name, value):
    setattr(settings, name, getattr(settings, name, value))

# Validate settings
if "%s.backends.ObjectPermBackend" % app_label not in settings.AUTHENTICATION_BACKENDS:
    raise ImproperlyConfigured("You have to set '%s.backends.ObjectPermBackend' to AUTHENTICATION_BACKENDS" % app_label)

# Set default settings
set_default(
    'OBJECT_PERMISSION_DEFAULT_HANDLER_CLASS', 
    'object_permission.handlers.authenticated.AuthenticatedObjectPermHandler')
set_default('OBJECT_PERMISSION_EXTRA_DEFAULT_PERMISSIONS', ['view'])
set_default('OBJECT_PERMISSION_BUILTIN_TEMPLATETAGS', True)
set_default('OBJECT_PERMISSION_AUTODISCOVER', True)
set_default('OBJECT_PERMISSION_HANDLER_MODULE_NAME', 'ophandler')
set_default('OBJECT_PERMISSION_DEPRECATED', False)

# Load site (this must be after the default settings has complete)
from sites import site

# Regist templatetags for ObjectPermission
if settings.OBJECT_PERMISSION_BUILTIN_TEMPLATETAGS:
    from django.template import add_to_builtins
    add_to_builtins('%s.templatetags.object_permission_tags' % app_label)

# Add extra default permission
def _get_all_permissions(opts):
    "Returns (codename, name) for all permissions in the given opts."
    perms = []
    for action in settings.OBJECT_PERMISSION_EXTRA_DEFAULT_PERMISSIONS:
        perms.append((_get_permission_codename(action, opts), u'Can %s %s' % (action, opts.verbose_name_raw)))
    return perms + list(opts.permissions)
def create_permissions(app, created_models, verbosity, **kwargs):
    from django.contrib.contenttypes.models import ContentType
    from django.contrib.auth.models import Permission
    app_models = get_models(app)
    if not app_models:
        return
    for klass in app_models:
        ctype = ContentType.objects.get_for_model(klass)
        for codename, name in _get_all_permissions(klass._meta):
            p, created = Permission.objects.get_or_create(codename=codename, content_type__pk=ctype.id,
                defaults={'name': name, 'content_type': ctype})
            if created and verbosity >= 2:
                print "Adding permission '%s'" % p
signals.post_syncdb.connect(create_permissions,
    dispatch_uid = "object_permission.management.create_permissions")

def autodiscover():
    """
    Auto-discover INSTALLED_APPS ophandler.py modules and fail silently when
    not present. This forces an import on them to register any handler bits they
    may want.
    """
    import copy
    from django.conf import settings
    from django.utils.importlib import import_module
    from django.utils.module_loading import module_has_submodule

    handler_module_name = settings.OBJECT_PERMISSION_HANDLER_MODULE_NAME

    for app in settings.INSTALLED_APPS:
        mod = import_module(app)
        # Attempt to import the app's object permission handler module.
        try:
            before_import_registry = copy.copy(site._registry)
            import_module('%s.%s' % (app, handler_module_name))
        except:
            # Reset the model registry to the state before the last import as
            # this import will have to reoccur on the next request and this
            # could raise NotRegistered and AlreadyRegistered exceptions
            site._registry = before_import_registry

            # Decide wheter to bubble up this error. If the app just
            # doesn't have an object permission module, we can ignore the error
            # attempting to import it, otherwise we want it to bubble up.
            if module_has_submodule(mod, handler_module_name):
                raise
if settings.OBJECT_PERMISSION_AUTODISCOVER:
    autodiscover()

if settings.OBJECT_PERMISSION_DEPRECATED:
    import warnings
    warnings.warn('deperecated future turnd on. you should turn off it with "OBJECT_PERMISSION_DEPRECATED"', DeprecationWarning)
    from deprecated.modify_object_permission import *
    from deprecated.mediators import ObjectPermissionMediator
    from deprecated.utils import generic_permission_check
    import utils
    utils.generic_permission_check = generic_permission_check



#!/usr/bin/env python
# vim: set fileencoding=utf8:
"""
utilities of object-permission


AUTHOR:
    lambdalisue[Ali su ae] (lambdalisue@hashnote.net)
    
Copyright:
    Copyright 2011 Alisue allright reserved.

License:
    Licensed under the Apache License, Version 2.0 (the "License"); 
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unliss required by applicable law or agreed to in writing, software
    distributed under the License is distrubuted on an "AS IS" BASICS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""
__AUTHOR__ = "lambdalisue (lambdalisue@hashnote.net)"
from ..utils import _get_object_from_date_based_object_detail
from ..utils import _get_object_from_list_detail_object_detail

def generic_permission_check(queryset, perm, request, *args, **kwargs):
    """
    Generic permission check

    useful to create custom permission_required decorator. DO NOT create your
    own permission_required while ``permission_required`` decorator is exsits
    at ``object_permission.decorators``
    
    Arguments:
        queryset    - queryset of object
        perm        - permission string
        request     - HttpRequest
        *args       - *args for view function
        **kwargs    - **kwargs for view function
    
    Note:
        This function is deprecated because it is for Functional generic view. 
        Getting object from parameters of functional generic view does not work
        sometimes because of the limitation.
    """
    if queryset:
        kwargs['queryset'] = queryset
        # functional based generic view
        if 'date_field' in kwargs:
            # assume that fn is date_based.object_detail
            obj = _get_object_from_date_based_object_detail(
                    request, *args, **kwargs)
        else:
            # assume that fn is list_detail.object_detail
            obj = _get_object_from_list_detail_object_detail(
                    request, *args, **kwargs)
    else:
        obj = None
    return request.user.has_perm(perm, obj)



#!/usr/bin/env python
# vim: set fileencoding=utf8:
"""
short module explanation


AUTHOR:
    lambdalisue[Ali su ae] (lambdalisue@hashnote.net)
    
Copyright:
    Copyright 2011 Alisue allright reserved.

License:
    Licensed under the Apache License, Version 2.0 (the "License"); 
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unliss required by applicable law or agreed to in writing, software
    distributed under the License is distrubuted on an "AS IS" BASICS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""
__AUTHOR__ = "lambdalisue (lambdalisue@hashnote.net)"
from django.contrib.contenttypes.models import ContentType
from django.contrib.auth.models import Permission
from django.contrib.auth.models import User
from django.contrib.auth.models import Group
from django.contrib.auth.models import AnonymousUser

from ..models import UserObjectPermission
from ..models import GroupObjectPermission
from ..models import AnonymousObjectPermission

class ObjectPermissionMediator(object):
    """object-permission mediator

    this is used to add/remove permissions of particular user (include
    anonymous) to particular object.

    >>> mediator = ObjectPermissionMediator
    >>> assert callable(getattr(mediator, 'get_permission'))
    >>> assert callable(getattr(mediator, 'contribute'))
    >>> assert callable(getattr(mediator, 'discontribute'))
    >>> assert callable(getattr(mediator, 'manager'))
    >>> assert callable(getattr(mediator, 'editor'))
    >>> assert callable(getattr(mediator, 'viewer'))
    >>> assert callable(getattr(mediator, 'reject'))

    This class is deprecated. Use ObjectPermMediator insted
    """

    @classmethod
    def get_permission(cls, obj, to=None):
        """Get permission for `obj` to `to` with parameter
        
        Attribute:
            obj        - tareget object for permission
            to         - User, Group, AnonymousUser to cotribute permission.
                        `None` mean contribute permission to all authenticated
                        user
        """
        
        ctype = ContentType.objects.get_for_model(obj)
        if isinstance(to, basestring) and to == 'anonymous':
            instance = AnonymousObjectPermission.objects.get_or_create(
                content_type=ctype,
                object_id=obj.pk)[0]
        elif isinstance(to, AnonymousUser):
            instance = AnonymousObjectPermission.objects.get_or_create(
                content_type=ctype,
                object_id=obj.pk)[0]
        elif isinstance(to, Group):
            instance = GroupObjectPermission.objects.get_or_create(
                group=to,
                content_type=ctype,
                object_id=obj.pk)[0]
        else:
            if isinstance(to, basestring):
                to = User.objects.get(username=to)
            instance = UserObjectPermission.objects.get_or_create(
                user=to,
                content_type=ctype,
                object_id=obj.pk)[0]
        return instance
    @classmethod
    def contribute(cls, obj, to=None, permissions=[], clear=False):
        """Contribute permission for `obj` to `to` with parameter
        
        Attribute:
            obj         - tareget object for permission
            to          - User, Group, AnonymousUser to cotribute permission.
                         `None` mean contribute permission to all authenticated
                         user
            permissions - codename list of permission
            append      - `True` to append `permissions`, default is `False`
        
        Notice:
            this method is super method. Use `manager`, `editor`, `viewer` 
            and `reject` for noraml usage.
        """
        ctype = ContentType.objects.get_for_model(obj)
        instance = cls.get_permission(obj, to)
        if clear:
            instance.permissions.clear()
        for perm in permissions:
            try:
                perm = perm.split('.')[-1]
            except IndexError:
                return False
            try:
                permission = Permission.objects.get(
                    content_type=ctype,
                    codename=perm,
                )
            except Permission.DoesNotExist:
                if hasattr(obj, 'object_permission_suffix'):
                    suffix = getattr(obj, 'object_permission_suffix')
                else:
                    # Django default permissions have model name at the end
                    suffix = "_" + str(ctype.model)
                perm = perm + suffix
                permission, created = Permission.objects.get_or_create(
                    content_type=ctype,
                    codename=perm,
                )
            instance.permissions.add(permission)
        instance.save()
        return instance
    @classmethod
    def discontribute(cls, obj, to=None, permissions=[]):
        """Discontribute permission for `obj` to `to` with parameter
        
        Attribute:
            obj         - tareget object for permission
            to          - User, Group, AnonymousUser to cotribute permission.
                         `None` mean contribute permission to all authenticated
                         user
            permissions - codename list of permission
        
        Notice:
            this method is super method. Use `manager`, `editor`, `viewer` 
            and `reject` for noraml usage.
        """
        ctype = ContentType.objects.get_for_model(obj)
        instance = cls.get_permission(obj, to)
        for perm in permissions:
            try:
                perm = perm.split('.')[-1]
            except IndexError:
                return False
            try:
                permission = Permission.objects.get(
                    content_type=ctype,
                    codename=perm,
                )
            except Permission.DoesNotExist:
                if hasattr(obj, 'object_permission_suffix'):
                    suffix = getattr(obj, 'object_permission_suffix')
                else:
                    # Django default permissions have model name at the end
                    suffix = "_" + str(ctype.model)
                perm = perm + suffix
                permission = Permission.objects.get(
                    content_type=ctype,
                    codename=perm,
                )
            instance.permissions.remove(permission)
        instance.save()
        return instance
    @classmethod
    def manager(cls, obj, to, extra_permissions=[]):
        """
        Contribute manager permission (can view, change, delete) for `obj` 
        to `to`
        
        Attribute:
            obj        - tareget object for permission
            to         - User, Group, AnonymousUser to cotribute permission.
                        `None` mean contribute permission to all authenticated
                        user
        """
        permissions = ['view', 'change', 'delete'] + list(extra_permissions)
        return cls.contribute(obj, to, permissions, clear=True)
    
    @classmethod
    def editor(cls, obj, to, extra_permissions=[]):
        """
        Contribute editor permission (can view, change. cannot delete) for 
        `obj` to `to`
        
        Attribute:
            obj        - tareget object for permission
            to         - User, Group, AnonymousUser to cotribute permission.
                        `None` mean contribute permission to all authenticated
                        user
        """
        permissions = ['view', 'change'] + list(extra_permissions)
        return cls.contribute(obj, to, permissions, clear=True)
    
    @classmethod
    def viewer(cls, obj, to, extra_permissions=[]):
        """
        Contribute viewer permission (can view. cannot change, delete) for 
        `obj` to `to`
        
        Attribute:
            obj        - tareget object for permission
            to         - User, Group, AnonymousUser to cotribute permission.
                        `None` mean contribute permission to all authenticated
                        user
        """
        permissions = ['view'] + list(extra_permissions)
        return cls.contribute(obj, to, permissions, clear=True)
    
    @classmethod
    def reject(cls, obj, to, extra_permissions=[]):
        """
        Contribute reject permission (cannot view change, delete) for `obj`
        to `to`
        
        Attribute:
            obj        - tareget object for permission
            to         - User, Group, AnonymousUser to cotribute permission.
                        `None` mean contribute permission to all authenticated
                        user
        """
        permissions = [] + list(extra_permissions)
        return cls.contribute(obj, to, permissions, clear=True)



#!/usr/bin/env python
# vim: set fileencoding=utf8:
"""
commands for django-object-permission

Django 1.2 template tag that supports {% elif %} branches and
'of' operator for checking object permission.


AUTHOR:
lambdalisue[Ali su ae] (lambdalisue@hashnote.net)
Copyright:
Copyright 2011 Alisue allright reserved.

License:
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unliss required by applicable law or agreed to in writing, software
distributed under the License is distrubuted on an "AS IS" BASICS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""
__AUTHOR__ = "lambdalisue (lambdalisue@hashnote.net)"
from django.conf import settings
from django.db.models.loading import get_models
from django.contrib.contenttypes.models import ContentType
from django.core.management.base import BaseCommand, CommandError
from django.core.exceptions import ImproperlyConfigured

from ...deprecated.mediators import ObjectPermissionMediator

class Command(BaseCommand):
    help = ("""Remodify object_permission of each model using model's """
            """`modify_object_permission(self, mediator, created)` function.""")

    def __init__(self, *args, **kwargs):
        import warnings
        warnings.warn(
                'deprecated: this command is deperecated. '
                'with new object-permission storategy, the command no '
                'longer required', DeprecationWarning)
        super(Command, self).__init__(*args, **kwargs)

    def handle(self, *app_labels, **options):
        from django.db import models
        if not app_labels:
            output = [self.handle_noargs(**options)]
        else:
            try:
                app_list = [models.get_app(app_label)
                        for app_label in app_labels]
            except (ImproperlyConfigured, ImportError), e:
                raise CommandError(
                        """"%s. Are you sure your INSTALLED_APPS setting is """
                        """correct?""" % e)
            output = []
            for app in app_list:
                app_output = self.handle_app(app, **options)
                if app_output:
                    output.append(app_output)
        return '\n'.join(output)
    
    def handle_app(self, app, **options):
        """
Remodify object_permission of each model using model's
`modify_object_permission(self, mediator, created)` function.
"""
        output = []
        model_list = get_models(app)
        for model in model_list:
            _output = self._remodify_object_permission(model, **options)
            if _output:
                output.append(_output)
        return "\n".join(output)
    
    def handle_noargs(self, **options):
        output = []
        content_types = ContentType.objects.all()
        for ct in content_types:
            model = ct.model_class()
            _output = self._remodify_object_permission(model, **options)
            if _output:
                output.append(_output)
        return "\n".join(output)
    
    def _remodify_object_permission(self, model, **options):
        NAME = settings.OBJECT_PERMISSION_MODIFY_FUNCTION
        NAME_M2M = settings.OBJECT_PERMISSION_MODIFY_M2M_FUNCTION
        output = []
        if not hasattr(model, NAME) and not hasattr(model, NAME_M2M):
            if options['verbosity'] == '2':
                output.append(
                        """Skipped: %s doesn't have `%s` and `%s` function."""
                        % (model, NAME, NAME_M2M))
        else:
            has = hasattr(model, NAME)
            has_m2m = hasattr(model, NAME_M2M)
            count = model.objects.count()
            for obj in model.objects.all():
                if has:
                    fn = getattr(obj, NAME)
                    fn(mediator=ObjectPermissionMediator, created=False)
                if has_m2m:
                    fn_m2m = getattr(obj, NAME_M2M)
                    for field in obj._meta.many_to_many:
                        sender = field.rel.through
                        model = field.rel.to
                        pk_set = [to.pk
                                for to in getattr(obj, field.attname).all()]
                        fn_m2m(mediator=ObjectPermissionMediator, sender=sender,
                               model=model, pk_set=pk_set, removed=False)
            if options['verbosity'] != '0':
                output.append(
                    """Modified: %s's object_permission has remodified (%d)"""
                    % (model, count))
        return "\n".join(output)



#!/usr/bin/env python
# vim: set fileencoding=utf8:
"""
short module explanation


AUTHOR:
    lambdalisue[Ali su ae] (lambdalisue@hashnote.net)
    
Copyright:
    Copyright 2011 Alisue allright reserved.

License:
    Licensed under the Apache License, Version 2.0 (the "License"); 
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unliss required by applicable law or agreed to in writing, software
    distributed under the License is distrubuted on an "AS IS" BASICS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""
__AUTHOR__ = "lambdalisue (lambdalisue@hashnote.net)"
from django.conf import settings
from django.db.models import Model

class AlreadyRegistered(Exception):
    pass
class NotRegistered(Exception):
    pass

class ObjectPermSite(object):
    """Manager class of ObjectPermHandler"""
    def __init__(self):
        self._registry = {}

    def register(self, model_or_iterable, handler_class=None):
        """registers the given model(s) with the given handler class.
        
        The model(s) should be Model classes, not instances.

        If a model is already registered, this will raise AlreadyRegistered
        """
        if handler_class is None:
            handler_class = settings.OBJECT_PERMISSION_DEFAULT_HANDLER_CLASS
        if issubclass(model_or_iterable, Model):
            model_or_iterable = [model_or_iterable]
        for model in model_or_iterable:
            if model in self._registry:
                raise AlreadyRegistered('The mode %s is already registered' % model.__name__)
            
            handler = handler_class(model)
            handler.bind()
            self._registry[model] = handler

    def unregister(self, model_or_iterable):
        """unregisters the given model(s).

        If a model isn't already registered, this will raise NotRegistered
        """
        if issubclass(model_or_iterable, Model):
            model_or_iterable = [model_or_iterable]
        for model in model_or_iterable:
            if model not in self._registry:
                raise NotRegistered('The model %s is not registered' % model.__name__)
            self._registry[model].unbind()
            del self._registry[model]

site = ObjectPermSite()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Author:       Alisue
# Last Change:  18-Mar-2011.
#
from setuptools import setup, find_packages

version = "0.2rc1"

def read(filename):
    import os.path
    return open(os.path.join(os.path.dirname(__file__), filename)).read()
setup(
    name="django-object-permission",
    version=version,
    description = "Add object specific permission for particualr User/Group, All authenticated user or Anonymous user",
    long_description=read('README.rst'),
    classifiers = [
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Programming Language :: Python',
        'Topic :: Internet :: WWW/HTTP',
    ],
    keywords = "django permission object",
    author = "Alisue",
    author_email = "lambdalisue@hashnote.net",
    url=r"https://github.com/lambdalisue/django-object-permission",
    download_url = r"https://github.com/lambdalisue/django-object-permission/tarball/master",
    license = 'BSD',
    packages = find_packages(),
    include_package_data = False,
    zip_safe = True,
    install_requires=[
        'setuptools',
        'django>=1.3',
        ],
)



#!/usr/bin/env python
# vim: set fileencoding=utf8:
"""
Object permission handler for model which has author field


AUTHOR:
    lambdalisue[Ali su ae] (lambdalisue@hashnote.net)
    
Copyright:
    Copyright 2011 Alisue allright reserved.

License:
    Licensed under the Apache License, Version 2.0 (the "License"); 
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unliss required by applicable law or agreed to in writing, software
    distributed under the License is distrubuted on an "AS IS" BASICS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""
__AUTHOR__ = "lambdalisue (lambdalisue@hashnote.net)"
try:
    from observer import watch
except ImportError:
    raise ImportError("AuthorObjectPermHandler required 'django-observer' installed")

from base import ObjectPermHandlerBase

class AuthorObjectPermHandler(ObjectPermHandlerBase):
    """ObjectPermHandler for model which has author field

    This handler contribute..

        1.  Manager permission to instance author
        2.  Viewer permission to authenticated user
        3.  Viewer permission to anonymous user if reject_anonymous is False

    """
    author_field = 'author'
    reject_anonymous = False

    def __del__(self):
        self.deleted()

    def unbind(self):
        super(AuthorObjectPermHandler, self).unbind()
        # Remove watcher
        self._author_field_watcher.unwatch()

    def get_author(self):
        return getattr(self.obj, self.author_field)

    def created(self):
        # Watch author change
        self._author_field_watcher = \
                watch(self.obj, self.author_field, self._author_field_changed)
        # Call updated method
        self.updated()

    def updated(self):
        # Author has full access
        self.mediator.manager(self.get_author())
        # Authenticated user can view
        self.mediator.viewer(None)
        if self.reject_anonymous:
            self.mediator.reject('anonymous')
        else:
            self.mediator.viewer('anonymous')

    def deleted(self):
        # Remove watcher
        self._author_field_watcher.unwatch()

    def _author_field_changed(self, sender, instance, attr):
        # reset all object permissions for obj
        self.mediator.reset()
        self.updated()



#!/usr/bin/env python
# vim: set fileencoding=utf8:
"""
models of django-object-permission


AUTHOR:
    lambdalisue[Ali su ae] (lambdalisue@hashnote.net)
    
Copyright:
    Copyright 2011 Alisue allright reserved.

License:
    Licensed under the Apache License, Version 2.0 (the "License"); 
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unliss required by applicable law or agreed to in writing, software
    distributed under the License is distrubuted on an "AS IS" BASICS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""
__AUTHOR__ = "lambdalisue (lambdalisue@hashnote.net)"
from django.db import models
from django.contrib.auth.models import User
from django.contrib.auth.models import Group
from django.contrib.auth.models import Permission
from django.contrib.contenttypes import generic
from django.contrib.contenttypes.models import ContentType
from django.utils.translation import ugettext_lazy as _
        
class BaseObjectPermissionManager(models.Manager):
    def get_for_model(self, obj):
        """get all object permissions for obj"""
        ct = ContentType.objects.get_for_model(obj)
        pk = obj.pk
        lookup_kwargs = {
                'content_type': ct,
                'object_id': pk,
            }
        return self.filter(**lookup_kwargs)
    def _get_filter_kwargs(self, obj):
        ct = ContentType.objects.get_for_model(obj)
        pk = obj.pk
        kwargs = {
                'content_type': ct,
                'object_id': pk
            }
        return kwargs
    def create_object_permission(self, obj, **kwargs):
        """create object permission"""
        kwargs = self._get_filter_kwargs(obj, **kwargs)
        return self.create(**kwargs)
    def get_or_create_object_permission(self, obj, **kwargs):
        """get or create object permission"""
        kwargs = self._get_filter_kwargs(obj, **kwargs)
        return self.get_or_create(**kwargs)

class BaseObjectPermission(models.Model):
    """
    Permission model for perticular object
    """
    content_type    = models.ForeignKey(
        ContentType, verbose_name=_('content type'), null=True)
    object_id       = models.PositiveIntegerField(_('object id'), null=True)
    content_object  = generic.GenericForeignKey()
    
    permissions     = models.ManyToManyField(
        Permission, verbose_name=_('permissions'), null=True)
    
    class Meta:
        abstract    = True
        
class GroupObjectPermissionManager(BaseObjectPermissionManager):
    def _get_filter_kwargs(self, obj, group):
        kwargs = super(GroupObjectPermissionManager, self)._get_filter_kwargs(obj)
        kwargs['group'] = group
        return kwargs
    def create_object_permission(self, obj, group):
        """create object permission"""
        return super(GroupObjectPermissionManager, self).create_object_permission(obj, group=group)
    def get_or_create_object_permission(self, obj, group):
        """get or create object permission"""
        return super(GroupObjectPermissionManager, self).get_or_create_object_permission(obj, group=group)

class GroupObjectPermission(BaseObjectPermission):
    """
    Object permission model for group
    """
    group   = models.ForeignKey(Group, verbose_name=_('group'))
    objects = GroupObjectPermissionManager()
    
    class Meta:
        ordering            = ('content_type', 'group')
        unique_together     = ('content_type', 'object_id', 'group')
        verbose_name        = _('group object permission')
        verbose_name_plural = _('group object permissions')
        
class AnonymousObjectPermissionManager(BaseObjectPermissionManager):
    def create_object_permission(self, obj):
        """create object permission"""
        return super(AnonymousObjectPermissionManager, self).create_object_permission(obj)
    def get_or_create_object_permission(self, obj):
        """get or create object permission"""
        return super(AnonymousObjectPermissionManager, self).get_or_create_object_permission(obj)

class AnonymousObjectPermission(BaseObjectPermission):
    """
    Object permission model for anonymous user
    """
    objects = AnonymousObjectPermissionManager()

    class Meta:
        ordering            = ('content_type',)
        unique_together     = ('content_type', 'object_id')
        verbose_name        = _('anonymous object permission')
        verbose_name_plural = _('anonymous object permissions')

class UserObjectPermissionManager(BaseObjectPermissionManager):
    def _get_filter_kwargs(self, obj, user):
        kwargs = super(UserObjectPermissionManager, self)._get_filter_kwargs(obj)
        kwargs['user'] = user
        return kwargs
    def create_object_permission(self, obj, user):
        """create object permission"""
        return super(UserObjectPermissionManager, self).create_object_permission(obj, user=user)
    def get_or_create_object_permission(self, obj, user):
        """get or create object permission"""
        return super(UserObjectPermissionManager, self).get_or_create_object_permission(obj, user=user)

class UserObjectPermission(BaseObjectPermission):
    """
    Object permission model for user
    """
    user    = models.ForeignKey(
        User, blank=True, null=True, 
        verbose_name=_('user'), 
        help_text=_('`None` for all authenticated user'))
    objects = UserObjectPermissionManager()
    
    class Meta:
        ordering            = ('content_type', 'user')
        unique_together     = ('content_type', 'object_id', 'user')
        verbose_name        = _('user object permission')
        verbose_name_plural = _('user object permissions')



#!/usr/bin/env python
# vim: set fileencoding=utf8:
"""
utilities of object-permission


AUTHOR:
    lambdalisue[Ali su ae] (lambdalisue@hashnote.net)
    
Copyright:
    Copyright 2011 Alisue allright reserved.

License:
    Licensed under the Apache License, Version 2.0 (the "License"); 
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unliss required by applicable law or agreed to in writing, software
    distributed under the License is distrubuted on an "AS IS" BASICS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""
__AUTHOR__ = "lambdalisue (lambdalisue@hashnote.net)"
import time
import datetime

from django.http import Http404
from django.db.models.fields import DateTimeField
from django.shortcuts import get_object_or_404

def get_perm_codename(perm):
    """get permission codename from django's standard permission format"""
    # Note:
    #   'app_label.permission_codename' is a standard permission format

    try:
        app, perm = perm.split('.', 1)
    except ValueError:
        # Non standard permission
        pass
    return perm

def get_perm_codename_with_suffix(perm, obj):
    """get permission codename suffix"""
    perm_codename = get_perm_codename(perm)
    if hasattr(obj, 'object_permission_suffix'):
        suffix = getattr(obj, 'object_permission_suffix')
    else:
        suffix = "_%s" % obj.__class__.__name__.lower()
    if perm_codename.endswith(suffix):
        return perm_codename
    return perm_codename + suffix
    
def _get_object_from_list_detail_object_detail(
        request, queryset, object_id=None, slug=None, 
        slug_field='slug', *args, **kwargs):
    """get object from parameters passed to list_detail.object_detail"""
    if object_id:
        obj = get_object_or_404(queryset, pk=object_id)
    elif slug and slug_field:
        obj = get_object_or_404(queryset, **{slug_field: slug})
    else:
        raise AttributeError(
                "Generic detail view must be called with either an "
                "object_id or a slug/slug_field.")
    return obj

def _get_object_from_date_based_object_detail(
        request, year, month, day, queryset, date_field,
        month_format='%b', day_format='%d', object_id=None,
        slug=None, slug_field='slug', **kwargs):
    """get object from parameters passed to date_based.object_detail"""
    from django.utils import timezone
    try:
        tt = time.strptime('%s-%s-%s' % (year, month, day), 
                            '%s-%s-%s' % ('%Y', month_format, day_format))
        date = datetime.date(*tt[:3])
    except ValueError:
        raise Http404

    model = queryset.model
    now = timezone.now()

    if isinstance(model._meta.get_field(date_field), DateTimeField):
        lookup_kwargs = {
                '%s__range' % date_field: (
                    datetime.datetime.combine(date, datetime.time.min),
                    datetime.datetime.combine(date, datetime.time.max)
                )}
    else:
        lookup_kwargs = {date_field: date}

    if date >= now.date() and not kwargs.get('allow_future', False):
        lookup_kwargs['%s__lte' % date_field] = now
    if object_id:
        lookup_kwargs['pk'] = object_id
    elif slug and slug_field:
        lookup_kwargs['%s__exact' % slug_field] = slug
    else:
        raise AttributeError(
                "Generic detail view must be called with either an "
                "object_id or a slug/slug_field.")
    return get_object_or_404(queryset, **lookup_kwargs)
